//	cc -I../../sendmail -I../../include -o sample sample.c libmilter.a ../libsm/libsm.a -pthread
//	% ./sample -p local:/var/run/f1.sock
/* auth-milter.c */
/* Version 0.10 2008.03.03 */
/* Local SMTP-AUTH checking */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <signal.h>
#include <sysexits.h>
#include <unistd.h>

#include "libmilter/mfapi.h"

#ifndef true
typedef int bool;
# define false	0
# define true	1
#endif /* ! true */

bool debug;
char *idString;


sfsistat
mlfi_envfrom(ctx, envfrom)
	SMFICTX *ctx;
	char **envfrom;
{
//  char *auth_author1 = smfi_getsymval(ctx, "{auth_author}");
  char *header_data = (char *)smfi_getpriv(ctx);
  if (header_data == NULL)
      {
      header_data = malloc(2048);
      if(header_data == NULL)
        {
        if(debug) syslog(LOG_NOTICE, "Can't allocate memory for header_data.");
        smfi_setreply(ctx, "550", "4.3.2", "Internal system error");
        return SMFIS_TEMPFAIL;
        }
      memset(header_data, 0, 2048);
      smfi_setpriv(ctx, (void *) header_data);
      }
														    
  if (smfi_getsymval(ctx, "{auth_authen}") != NULL)
    {
    sprintf (header_data, "authenticated user=%s.%s, %s\0", idString,
             smfi_getsymval(ctx, "{auth_authen}"), smfi_getsymval(ctx, "{auth_type}"));
    if(debug) syslog (LOG_NOTICE, header_data);
    }
  return SMFIS_CONTINUE;
}


sfsistat
mlfi_eom(ctx)
	SMFICTX *ctx;
{
  char *var = (char *)smfi_getpriv(ctx);
  if(var == NULL) return;
  if(strlen(var) > 1)
    smfi_addheader(ctx, "X-SMTP-AUTH-ByMe\0", var);
  return SMFIS_CONTINUE;
}

sfsistat
mlfi_close(ctx)
	SMFICTX *ctx;
{
  char *var = (char *)smfi_getpriv(ctx);
  if(var != NULL) free(var);
  if(debug) syslog(LOG_NOTICE, "Free \'header_data\'.");
  smfi_setpriv(ctx, NULL);     
//  if(idString != NULL) free(idString);
  if(debug) syslog(LOG_NOTICE, "Work was complete.");
  return SMFIS_CONTINUE;
}

sfsistat
mlfi_abort(ctx)
	SMFICTX *ctx;
{
	return 0;
}


struct smfiDesc smfilter =
{
	"Local SMTP AUTH Checker",	/* filter name */
	SMFI_VERSION,	/* version code -- do not change */
	SMFIF_ADDHDRS,	/* flags */
	NULL,		/* connection info filter */
	NULL,		/* SMTP HELO command filter */
	mlfi_envfrom,	/* envelope sender filter */
	NULL,		/* envelope recipient filter */
	NULL,		/* header filter */
	NULL,		/* end of header */
	NULL,		/* body block filter */
	mlfi_eom,	/* end of message */
	NULL,		/* message aborted */
	mlfi_close	/* connection cleanup */
};

void term_handler(int i)
  {
  if(idString) free(idString);
  if(debug) syslog(LOG_NOTICE, "Bye-bye...");
  exit(0);
  }


int
main(argc, argv)
	int argc;
	char *argv[];
{
//    
int c;

bool conntosendmail = false;
debug = false;

// :        
openlog(NULL, LOG_PID, LOG_MAIL|LOG_USER);

idString = malloc(2048);
if(!idString)
  {
  if(debug) syslog(LOG_NOTICE, "Can't alloc memory for \'idString\' variable");
  exit(1);
  }

// c -  
// h - 
// d -  
// i -  
	const char *args = "c:i:hd";

	/* Process command line options */
	while ((c = getopt(argc, argv, args)) != -1)
	{
		switch (c)
		{
		  case 'c':
			if (optarg == NULL || *optarg == '\0')
			{
				(void) fprintf(stderr, "Illegal conn: %s\n",
					       optarg);
				exit(EX_USAGE);
			}
                        conntosendmail = true;
			char *ofile;
			if (!strncmp(optarg, "unix:", 5))
        		    ofile = optarg + 5;
	    		else
		    	    if (!strncmp(optarg, "local:", 6)) ofile = optarg + 6;
			if (ofile) unlink(ofile);
			(void) smfi_setconn(optarg);
			break;
		  case 'i':
			if (optarg == NULL || *optarg == '\0')
			{
				(void) fprintf(stderr, "Illegal idString: %s\n",
					       optarg);
				exit(EX_USAGE);
			}
			strcpy(idString, optarg);
			break;
		  case 'h':
			printf ("Usage: auth-milter -c {unix|local|inet}:path [-i string] [-d] [-h] \n");
			printf ("Options:\n");
			printf ("   -c {unix|local|inet}:path - path to socket file or network address to connect to sendmail\n");
			printf ("   -d                        - debug mode\n");
			printf ("   -i string                 - id string\n");
			printf ("   -h                        - this help\n");
			exit(EX_USAGE);
			break;
		  case 'd':
			debug = true;
			break;
		}
	}

if (!conntosendmail)
  {
  printf ("Usage: auth-milter -c {unix|local|inet}:socket_path [-i string] [-d] [-h]\n");
  exit(EX_USAGE);
  }

//    
//load_localdomains_list();

//  
struct sigaction sa;
sigset_t newset;
sigemptyset(&newset);
sigaddset(&newset, SIGHUP);
sigprocmask(SIG_BLOCK, &newset, 0);
sa.sa_handler = term_handler;
sigaction(SIGTERM, &sa, 0);

//
    if (daemon(0, 0)) {
        syslog(LOG_NOTICE, "daemonize failed.");
//        syslog(LOG_NOTICE, "daemonize failed: %s", strerror(errno));
        }
//     	
	smfi_settimeout(350);

//  
	if (smfi_register(smfilter) == MI_FAILURE)
	{
		syslog(LOG_NOTICE, "smfi_register failed");
		exit(EX_UNAVAILABLE);
	}
   return smfi_main();
}

/* eof */

//$Revision: 0.23 $, Last updated $Date: 2007/08/06 12:29:00 $

